@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-xine.c,v 1.188 2005/05/22 22:44:02 hadess Exp $
+ * $Id: bacon-video-widget-xine.c,v 1.199 2005/07/12 19:39:36 hadess Exp $
  *
  * the xine engine in a widget - implementation
  */
@@ -139,6 +139,7 @@ struct BaconVideoWidgetPrivate {
 	Display *display;
 	int screen;
 	GdkWindow *video_window;
+	GdkCursor *cursor;
 
 	/* Visual effects */
 	char *vis_name;
@@ -454,6 +455,9 @@ bacon_video_widget_finalize (GObject *object)
 	if (bvw->priv->xine != NULL) {
 		xine_exit (bvw->priv->xine);
 	}
+	if (bvw->priv->cursor != NULL) {
+		gdk_cursor_unref (bvw->priv->cursor);
+	}
 	g_free (bvw->priv->vis_name);
 	g_free (bvw->priv->mediadev);
 	g_object_unref (G_OBJECT (bvw->priv->gc));
@@ -466,8 +470,7 @@ bacon_video_widget_finalize (GObject *object)
 	g_async_queue_unref (bvw->priv->queue);
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 
-	bvw->priv = NULL;
-	bvw = NULL;
+	g_free (bvw->priv);
 }
 
 static void
@@ -1216,6 +1219,7 @@ bacon_video_widget_idle_signal (BaconVideoWidget *bvw)
 		g_signal_emit (G_OBJECT (bvw),
 				bvw_table_signals[ERROR], 0,
 				data->msg, TRUE, data->fatal);
+		break;
 	default:
 		g_assert_not_reached ();
 	}
@@ -1309,6 +1313,15 @@ xine_event_message (BaconVideoWidget *bvw, xine_ui_message_data_t *data)
 		else
 			message = g_strdup (_("The server refused access to this file or stream."));
 		break;
+
+/* FIXME missing stuff from libxine includes (<= 1.1.x) */
+#ifndef XINE_MSG_FILE_EMPTY
+#define XINE_MSG_FILE_EMPTY 13
+#endif
+	case XINE_MSG_FILE_EMPTY:
+		num = BVW_ERROR_EMPTY_FILE;
+		message  = g_strdup (_("The file you tried to play is an empty file."));
+		break;
 	default:
 		D("xine_event_message: unhandled error\ntype: %d", data->type);
 		return;
@@ -1338,14 +1351,17 @@ xine_event (void *user_data, const xine_event_t *event)
 		spubtn = (xine_spu_button_t *) event->data;
 		if (spubtn->direction)
 		{
-			GdkCursor *cursor;
-			cursor = gdk_cursor_new (GDK_HAND2);
-			gdk_window_set_cursor
-				(bvw->priv->video_window, cursor);
-			gdk_cursor_unref (cursor);
+			if (bvw->priv->cursor == NULL) {
+				bvw->priv->cursor = gdk_cursor_new (GDK_HAND2);
+			}
 		} else {
-			gdk_window_set_cursor(bvw->priv->video_window, NULL);
+			if (bvw->priv->cursor != NULL) {
+				gdk_cursor_unref (bvw->priv->cursor);
+				bvw->priv->cursor = NULL;
+			}
 		}
+		gdk_window_set_cursor (bvw->priv->video_window,
+				bvw->priv->cursor);
 		break;
 	case XINE_EVENT_UI_PLAYBACK_FINISHED:
 		if (bvw->priv->got_redirect != FALSE)
@@ -1511,11 +1527,6 @@ bacon_video_widget_unrealize (GtkWidget *widget)
         nvtv_simple_exit();
 #endif
 
-
-/* FIXME missing stuff from libxine includes (<= 1-rc7) */
-#ifndef XINE_GUI_SEND_WILL_DESTROY_DRAWABLE
-#define XINE_GUI_SEND_WILL_DESTROY_DRAWABLE 9
-#endif
 	xine_port_send_gui_data (bvw->priv->vo_driver,
 			XINE_GUI_SEND_WILL_DESTROY_DRAWABLE,
 			(void*)bvw->priv->video_window);
@@ -1939,6 +1950,7 @@ bacon_video_widget_open_with_subtitle (BaconVideoWidget *bvw, const char *mrl,
 		const char *subtitle_uri, GError **error)
 {
 	int err;
+	const char *layer;
 
 	g_return_val_if_fail (mrl != NULL, FALSE);
 	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
@@ -1964,7 +1976,9 @@ bacon_video_widget_open_with_subtitle (BaconVideoWidget *bvw, const char *mrl,
 		return FALSE;
 	}
 
-	if (strcmp (xine_get_meta_info (bvw->priv->stream, XINE_META_INFO_SYSTEMLAYER), "MNG") == 0 && bvw->priv->logo_mode == FALSE)
+	layer = xine_get_meta_info (bvw->priv->stream,
+			XINE_META_INFO_SYSTEMLAYER);
+	if ((strcmp (layer, "MNG") == 0 || strcmp (layer, "imagedmx") == 0)&& bvw->priv->logo_mode == FALSE)
 	{
 		bacon_video_widget_close (bvw);
 		g_set_error (error, BVW_ERROR, BVW_ERROR_STILL_IMAGE,
@@ -2142,7 +2156,15 @@ gboolean bacon_video_widget_seek_time (BaconVideoWidget *bvw, gint64 time,
 		return TRUE;
 	}
 
-	length = bacon_video_widget_get_stream_length (bvw);
+	if (time > length) {
+		signal_data *data;
+
+		data = g_new0 (signal_data, 1);
+		data->signal = EOS_ASYNC;
+		g_async_queue_push (bvw->priv->queue, data);
+		g_idle_add ((GSourceFunc) bacon_video_widget_idle_signal, bvw);
+		return TRUE;
+	}
 
 	error = xine_play (bvw->priv->stream, 0, CLAMP (time, 0, length));
 
@@ -2483,9 +2505,10 @@ bacon_video_widget_set_show_cursor (BaconVideoWidget *bvw,
 
 	if (show_cursor == FALSE)
 	{
-		eel_gdk_window_set_invisible_cursor (bvw->priv->video_window);
+		totem_gdk_window_set_invisible_cursor (bvw->priv->video_window);
 	} else {
-		gdk_window_set_cursor (bvw->priv->video_window, NULL);
+		gdk_window_set_cursor (bvw->priv->video_window,
+				bvw->priv->cursor);
 	}
 
 	bvw->priv->cursor_shown = show_cursor;
@@ -3099,6 +3122,9 @@ bacon_video_widget_set_zoom (BaconVideoWidget *bvw, int zoom)
 	g_return_if_fail (bvw->priv->xine != NULL);
 	g_return_if_fail (zoom >= 0 && zoom <= 400);
 
+	if (bvw->priv->stream == NULL)
+		return;
+
 	xine_set_param (bvw->priv->stream,
 			XINE_PARAM_VO_ZOOM_X, zoom);
 	xine_set_param (bvw->priv->stream,
@@ -3112,6 +3138,9 @@ bacon_video_widget_get_zoom (BaconVideoWidget *bvw)
 	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), 100);
 	g_return_val_if_fail (bvw->priv->xine != NULL, 100);
 
+	if (bvw->priv->stream == NULL)
+		return 100;
+
 	return xine_get_param (bvw->priv->stream, XINE_PARAM_VO_ZOOM_X);
 }
 
@@ -3328,6 +3357,17 @@ bacon_video_widget_get_metadata_int (BaconVideoWidget *bvw,
 		integer = xine_get_stream_info (bvw->priv->stream,
 				XINE_STREAM_INFO_VIDEO_BITRATE) / 1000;
 		break;
+	 case BVW_INFO_TRACK_NUMBER:
+		{
+			const char *string;
+			string = xine_get_meta_info (bvw->priv->stream,
+					XINE_META_INFO_TRACK_NUMBER);
+			if (string == NULL)
+				return;
+
+			integer = (int) g_ascii_strtod (string, NULL);
+		}
+		break;
 	 default:
 		g_assert_not_reached ();
 	 }
@@ -3396,6 +3436,7 @@ bacon_video_widget_get_metadata (BaconVideoWidget *bvw,
 	case BVW_INFO_FPS:
 	case BVW_INFO_AUDIO_BITRATE:
 	case BVW_INFO_VIDEO_BITRATE:
+	case BVW_INFO_TRACK_NUMBER:
 		bacon_video_widget_get_metadata_int (bvw, type, value);
 		break;
 	case BVW_INFO_HAS_VIDEO:
@@ -3624,6 +3665,23 @@ bacon_video_widget_get_current_frame (BaconVideoWidget *bvw)
 		return NULL;
 	}
 
+	switch (ratio) {
+	case XINE_VO_ASPECT_SQUARE:
+		ratio = 10000.0;
+		break;
+	case XINE_VO_ASPECT_4_3:
+		ratio = 10000.0 * 4 / 3;
+		break;
+	case XINE_VO_ASPECT_ANAMORPHIC:
+		ratio = 10000.0 * 16 / 9;
+		break;
+	case XINE_VO_ASPECT_DVB:
+		ratio = 10000.0 * 2.11;
+		break;
+	default:
+		ratio = 0.0;
+	}
+
 	/* Convert to rgb */
 	rgb = yv12torgb (y, u, v, width, height);
 
@@ -3632,10 +3690,6 @@ bacon_video_widget_get_current_frame (BaconVideoWidget *bvw)
 			8, width, height, 3 * width,
 			(GdkPixbufDestroyNotify) g_free, NULL);
 
-	/* MPEG streams have ratio information */
-	ratio = xine_get_stream_info (bvw->priv->stream,
-			XINE_STREAM_INFO_VIDEO_RATIO);
-
 	if (ratio != 10000.0 && ratio != 0.0)
 	{
 		GdkPixbuf *tmp;
