@@ -27,13 +27,58 @@
 #include <glib/gi18n.h>
 #include <glade/glade.h>
 #include <string.h>
+#include "video-utils.h"
+#include "totem-interface.h"
 
 #include "debug.h"
 
+/* used in bacon_video_widget_properties_update() */
+#define UPDATE_FROM_STRING(type, name) \
+	do { \
+		const char *temp; \
+		bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw), \
+						 type, &value); \
+		if ((temp = g_value_get_string (&value)) != NULL) { \
+			bacon_video_widget_properties_set_label (props, name, \
+								 temp); \
+		} \
+		g_value_unset (&value); \
+	} while (0)
+
+#define UPDATE_FROM_INT(type, name, format) \
+	do { \
+		char *temp; \
+		bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw), \
+						 type, &value); \
+		temp = g_strdup_printf (gettext (format), \
+					g_value_get_int (&value)); \
+		bacon_video_widget_properties_set_label (props, name, temp); \
+		g_free (temp); \
+		g_value_unset (&value); \
+	} while (0)
+
+#define UPDATE_FROM_INT2(type1, type2, name, format) \
+	do { \
+		int x, y; \
+		char *temp; \
+		bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw), \
+						 type1, &value); \
+		x = g_value_get_int (&value); \
+		g_value_unset (&value); \
+		bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw), \
+						 type2, &value); \
+		y = g_value_get_int (&value); \
+		g_value_unset (&value); \
+		temp = g_strdup_printf (gettext (format), x, y); \
+		bacon_video_widget_properties_set_label (props, name, temp); \
+		g_free (temp); \
+	} while (0)
+
 struct BaconVideoWidgetPropertiesPrivate
 {
 	GladeXML *xml;
 	GtkWidget *vbox;
+	int time;
 };
 
 static GtkWidgetClass *parent_class = NULL;
@@ -67,48 +112,6 @@ bacon_video_widget_properties_finalize (GObject *object)
 	}
 }
 
-char *
-bacon_video_widget_properties_time_to_string (int time)
-{
-	char *secs, *mins, *hours, *string;
-	int sec, min, hour;
-
-	sec = time % 60;
-	time = time - sec;
-	min = (time % (60*60)) / 60;
-	time = time - (min * 60);
-	hour = time / (60*60);
-
-	hours = g_strdup_printf (ngettext ("%d hour", "%d hours", hour), hour);
-
-	mins = g_strdup_printf (ngettext ("%d minute",
-					  "%d minutes", min), min);
-
-	secs = g_strdup_printf (ngettext ("%d second",
-					  "%d seconds", sec), sec);
-
-	if (hour > 0)
-	{
-		/* hour:minutes:seconds */
-		string = g_strdup_printf (_("%s %s %s"), hours, mins, secs);
-	} else if (min > 0) {
-		/* minutes:seconds */
-		string = g_strdup_printf (_("%s %s"), mins, secs);
-	} else if (sec > 0) {
-		/* seconds */
-		string = g_strdup_printf (_("%s"), secs);
-	} else {
-		/* 0 seconds */
-		string = g_strdup (_("0 seconds"));
-	}
-
-	g_free (hours);
-	g_free (mins);
-	g_free (secs);
-
-	return string;
-}
-
 static void
 bacon_video_widget_properties_set_label (BaconVideoWidgetProperties *props,
 			       const char *name, const char *text)
@@ -119,11 +122,16 @@ bacon_video_widget_properties_set_label (BaconVideoWidgetProperties *props,
 	gtk_label_set_text (GTK_LABEL (item), text);
 }
 
-static void
+void
 bacon_video_widget_properties_reset (BaconVideoWidgetProperties *props)
 {
 	GtkWidget *item;
 
+	g_return_if_fail (props != NULL);
+	g_return_if_fail (BACON_IS_VIDEO_WIDGET_PROPERTIES (props));
+
+	item = glade_xml_get_widget (props->priv->xml, "video_vbox");
+	gtk_widget_show (item);
 	item = glade_xml_get_widget (props->priv->xml, "video");
 	gtk_widget_set_sensitive (item, FALSE);
 	item = glade_xml_get_widget (props->priv->xml, "audio");
@@ -133,140 +141,131 @@ bacon_video_widget_properties_reset (BaconVideoWidgetProperties *props)
 	bacon_video_widget_properties_set_label (props, "title", _("Unknown"));
 	/* Artist */
 	bacon_video_widget_properties_set_label (props, "artist", _("Unknown"));
+	/* Album */
+	bacon_video_widget_properties_set_label (props, "album", _("Unknown"));
 	/* Year */
 	bacon_video_widget_properties_set_label (props, "year", _("Unknown"));
 	/* Duration */
-	bacon_video_widget_properties_set_label (props, "duration", _("0 second"));
+	bacon_video_widget_properties_from_time (props, 0);
+
 	/* Dimensions */
 	bacon_video_widget_properties_set_label (props, "dimensions", _("0 x 0"));
 	/* Video Codec */
 	bacon_video_widget_properties_set_label (props, "vcodec", _("N/A"));
+	/* Video Bitrate */
+	bacon_video_widget_properties_set_label (props, "video_bitrate",
+			_("0 kbps"));
 	/* Framerate */
 	bacon_video_widget_properties_set_label (props, "framerate",
 			_("0 frames per second"));
-	/* Bitrate */
-	bacon_video_widget_properties_set_label (props, "bitrate", _("0 kbps"));
+	/* Audio Bitrate */
+	bacon_video_widget_properties_set_label (props, "audio_bitrate",
+			_("0 kbps"));
 	/* Audio Codec */
 	bacon_video_widget_properties_set_label (props, "acodec", _("N/A"));
 }
 
-static void
-bacon_video_widget_properties_set_from_current
-(BaconVideoWidgetProperties *props, BaconVideoWidget *bvw)
+void
+bacon_video_widget_properties_from_time (BaconVideoWidgetProperties *props,
+					 int time)
+{
+	char *string;
+
+	g_return_if_fail (props != NULL);
+	g_return_if_fail (BACON_IS_VIDEO_WIDGET_PROPERTIES (props));
+
+	if (time == props->priv->time)
+		return;
+
+	string = totem_time_to_string_text (time);
+	bacon_video_widget_properties_set_label (props, "duration", string);
+	g_free (string);
+
+	props->priv->time = time;
+}
+
+void
+bacon_video_widget_properties_update (BaconVideoWidgetProperties *props,
+				      BaconVideoWidget *bvw)
 {
 	GtkWidget *item;
 	GValue value = { 0, };
-	char *string;
-	int x, y;
+	gboolean has_type;
+
+	g_return_if_fail (props != NULL);
+	g_return_if_fail (BACON_IS_VIDEO_WIDGET_PROPERTIES (props));
+	g_return_if_fail (bvw != NULL);
 
 	/* General */
-	bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw),
-			BVW_INFO_TITLE, &value);
-	bacon_video_widget_properties_set_label (props, "title",
-			g_value_get_string (&value)
-			? g_value_get_string (&value)
-			: _("Unknown"));
-	g_value_unset (&value);
-
-	bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw),
-			BVW_INFO_ARTIST, &value);
-	bacon_video_widget_properties_set_label (props, "artist",
-			g_value_get_string (&value)
-			? g_value_get_string (&value) : _("Unknown"));
-	g_value_unset (&value);
-
-	bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw),
-			BVW_INFO_YEAR, &value);
-	bacon_video_widget_properties_set_label (props, "year",
-			g_value_get_string (&value)
-			? g_value_get_string (&value) : _("Unknown"));
-	g_value_unset (&value);
+	UPDATE_FROM_STRING (BVW_INFO_TITLE, "title");
+	UPDATE_FROM_STRING (BVW_INFO_ARTIST, "artist");
+	UPDATE_FROM_STRING (BVW_INFO_ALBUM, "album");
+	UPDATE_FROM_STRING (BVW_INFO_YEAR, "year");
 
 	bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw),
 			BVW_INFO_DURATION, &value);
-	string = bacon_video_widget_properties_time_to_string
-		(g_value_get_int (&value));
-	bacon_video_widget_properties_set_label (props, "duration", string);
-	g_free (string);
+	bacon_video_widget_properties_from_time (props,
+			g_value_get_int (&value) * 1000);
 	g_value_unset (&value);
 
 	/* Video */
 	item = glade_xml_get_widget (props->priv->xml, "video");
 	bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw),
 			BVW_INFO_HAS_VIDEO, &value);
-	if (g_value_get_boolean (&value) == FALSE)
-		gtk_widget_set_sensitive (item, FALSE);
-	else
-		gtk_widget_set_sensitive (item, TRUE);
+	has_type = g_value_get_boolean (&value);
+	gtk_widget_set_sensitive (item, has_type);
 	g_value_unset (&value);
 
-	bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw),
-			BVW_INFO_DIMENSION_X, &value);
-	x = g_value_get_int (&value);
-	g_value_unset (&value);
-	bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw),
-			BVW_INFO_DIMENSION_Y, &value);
-	y = g_value_get_int (&value);
-	g_value_unset (&value);
-	string = g_strdup_printf ("%d x %d", x, y);
-	bacon_video_widget_properties_set_label (props, "dimensions", string);
-	g_free (string);
+	item = glade_xml_get_widget (props->priv->xml, "video_vbox");
 
-	bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw),
-			BVW_INFO_VIDEO_CODEC, &value);
-	bacon_video_widget_properties_set_label (props, "vcodec",
-			g_value_get_string (&value)
-			? g_value_get_string (&value) : _("N/A"));
-	g_value_unset (&value);
-
-	bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw),
-			BVW_INFO_FPS, &value);
-	string = g_strdup_printf (_("%d frames per second"),
-			g_value_get_int (&value));
-	bacon_video_widget_properties_set_label (props, "framerate", string);
-	g_free (string);
-	g_value_unset (&value);
+	if (has_type != FALSE)
+	{
+		UPDATE_FROM_INT2 (BVW_INFO_DIMENSION_X, BVW_INFO_DIMENSION_Y,
+				  "dimensions", N_("%d x %d"));
+		UPDATE_FROM_STRING (BVW_INFO_VIDEO_CODEC, "vcodec");
+		UPDATE_FROM_INT (BVW_INFO_FPS, "framerate",
+				 N_("%d frames per second"));
+		UPDATE_FROM_INT (BVW_INFO_VIDEO_BITRATE, "video_bitrate",
+				 N_("%d kbps"));
+	} else {
+		gtk_widget_hide (item);
+	}
 
 	/* Audio */
 	item = glade_xml_get_widget (props->priv->xml, "audio");
 	bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw),
 			BVW_INFO_HAS_AUDIO, &value);
-	if (g_value_get_boolean (&value) == FALSE)
-		gtk_widget_set_sensitive (item, FALSE);
-	else
-		gtk_widget_set_sensitive (item, TRUE);
+	has_type = g_value_get_boolean (&value);
+	gtk_widget_set_sensitive (item, has_type);
 	g_value_unset (&value);
 
-	bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw),
-			BVW_INFO_BITRATE, &value);
-	string = g_strdup_printf (_("%d kbps"), g_value_get_int (&value));
-	bacon_video_widget_properties_set_label (props, "bitrate", string);
-	g_free (string);
-	g_value_unset (&value);
+	if (has_type != FALSE)
+	{
+		UPDATE_FROM_INT (BVW_INFO_AUDIO_BITRATE, "audio_bitrate",
+				 N_("%d kbps"));
+		UPDATE_FROM_STRING (BVW_INFO_AUDIO_CODEC, "acodec");
+	}
 
-	bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw),
-			BVW_INFO_AUDIO_CODEC, &value);
-	bacon_video_widget_properties_set_label (props, "acodec",
-			g_value_get_string (&value)
-			? g_value_get_string (&value) : _("N/A"));
-	g_value_unset (&value);
+#undef UPDATE_FROM_STRING
+#undef UPDATE_FROM_INT
+#undef UPDATE_FROM_INT2
 }
 
 void
-bacon_video_widget_properties_update (BaconVideoWidgetProperties *props,
-		BaconVideoWidget *bvw,
-		gboolean reset)
+bacon_video_widget_properties_from_metadata (BaconVideoWidgetProperties *props,
+					     const char *title,
+					     const char *artist,
+					     const char *album)
 {
 	g_return_if_fail (props != NULL);
 	g_return_if_fail (BACON_IS_VIDEO_WIDGET_PROPERTIES (props));
+	g_return_if_fail (title != NULL);
+	g_return_if_fail (artist != NULL);
+	g_return_if_fail (album != NULL);
 
-	if (reset != FALSE)
-	{
-		bacon_video_widget_properties_reset (props);
-	} else {
-		g_return_if_fail (bvw != NULL);
-		bacon_video_widget_properties_set_from_current (props, bvw);
-	}
+	bacon_video_widget_properties_set_label (props, "title", title);
+	bacon_video_widget_properties_set_label (props, "artist", artist);
+	bacon_video_widget_properties_set_label (props, "album", album);
 }
 
 GtkWidget*
@@ -274,14 +273,10 @@ bacon_video_widget_properties_new (void)
 {
 	BaconVideoWidgetProperties *props;
 	GladeXML *xml;
-	char *filename;
 	GtkWidget *vbox;
 
-	filename = g_build_filename (DATADIR,
-			"totem", "properties.glade", NULL);
-
-	xml = glade_xml_new (filename, "vbox1", NULL);
-	g_free (filename);
+	xml = totem_interface_load_with_root ("properties.glade",
+			"vbox1", _("Properties dialog"), TRUE, NULL);
 
 	if (xml == NULL)
 		return NULL;
@@ -293,7 +288,7 @@ bacon_video_widget_properties_new (void)
 	vbox = glade_xml_get_widget (props->priv->xml, "vbox1");
 	gtk_box_pack_start (GTK_BOX (props), vbox, TRUE, TRUE, 0);
 
-	bacon_video_widget_properties_update (props, NULL, TRUE);
+	bacon_video_widget_properties_reset (props);
 
 	gtk_widget_show_all (GTK_WIDGET (props));
 
