@@ -64,6 +64,8 @@ struct Totem {
 	BaconVideoWidget *bvw;
 	GtkWidget *prefs;
 	GtkWidget *statusbar;
+	GtkWidget *sidebar;
+	gboolean sidebar_shown;
 
 	/* Separate Dialogs */
 	GtkWidget *properties;
@@ -83,7 +85,6 @@ struct Totem {
 
 	/* Volume */
 	GtkWidget *volume;
-	GtkAdjustment *voladj;
 	gboolean vol_lock;
 	gboolean vol_fs_lock;
 	gfloat prev_volume;
