@@ -25,6 +25,9 @@
 #include <glib.h>
 #include <string.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
 #include "totem-uri.h"
 #include "totem-private.h"
@@ -56,6 +59,29 @@ totem_is_media (const char *uri)
 	return FALSE;
 }
 
+gboolean
+totem_is_block_device (const char *uri)
+{
+	struct stat buf;
+	char *local;
+
+	if (uri == NULL)
+		return FALSE;
+
+	if (g_str_has_prefix (uri, "file:") == FALSE)
+		return FALSE;
+	local = g_filename_from_uri (uri, NULL, NULL);
+	if (local == NULL)
+		return FALSE;
+	if (stat (local, &buf) != 0) {
+		g_free (local);
+		return FALSE;
+	}
+	g_free (local);
+
+	return (S_ISBLK (buf.st_mode));
+}
+
 char*
 totem_create_full_path (const char *path)
 {
@@ -161,6 +187,35 @@ totem_uri_get_subtitle_uri (const char *uri)
 		return subtitle;
 	}
 
+	g_free (subtitle);
 	return NULL;
 }
 
+char*
+totem_uri_escape_for_display (const char *uri)
+{
+	char *disp, *tmp;
+
+	disp = gnome_vfs_unescape_string_for_display (uri);
+	/* If we don't have UTF-8, try to convert */
+	if (g_utf8_validate (disp, -1, NULL) != FALSE)
+		return disp;
+
+	/* If we don't have UTF-8, try to convert */
+	tmp = g_locale_to_utf8 (disp, -1, NULL, NULL, NULL);
+	/* If we couldn't convert using the current codeset, try
+	 * another one */
+	if (tmp != NULL) {
+		g_free (disp);
+		return tmp;
+	}
+
+	tmp = g_convert (disp, -1, "UTF-8", "ISO8859-1", NULL, NULL, NULL);
+	if (tmp != NULL) {
+		g_free (disp);
+		return tmp;
+	}
+
+	return g_strdup (uri);
+}
+
